@@ -48,7 +48,8 @@
 >>> vars = {}
 >>> cfg_file = StringIO.StringIO('\\n'.join(cfg_lines))
 >>> result = config.file_config(vars, cfg_file)
->>> assert result == [cfg_file]
+>>> result == [cfg_file]
+True
 >>> vars['options']['host']
 'example.com'
 >>> vars['platform'].clients[0].user
@@ -62,8 +63,20 @@
 >>> vars['formatters'][0].count
 >>> vars['formatters'][0].delimiter
 u' | '
+>>> import os, tempfile
+>>> cfg_fd, cfg_file = tempfile.mkstemp()
+>>> os.close(cfg_fd)
+>>> open(cfg_file, 'w').write('\\n'.join(cfg_lines))
+>>> result = config.file_config(vars, cfg_file)
+>>> result == [cfg_file]
+True
+>>> os.unlink(cfg_file)
+>>> result = config.file_config(vars, cfg_file)
+>>> result == []
+True
 """
 
+import sys
 import ConfigParser
 import shlex
 
@@ -85,6 +98,14 @@ def file_config(vars, fname, defaults = None, prefix = '',
             result = [fname]
         else:
             result = cp.read(fname)
+            if sys.version_info < (2, 4):
+                if cp.sections():
+                    if type(fname) == list:
+                        result = fname
+                    else:
+                        result = [fname]
+                else:
+                    result = []
     except (ConfigParser.MissingSectionHeaderError,
             ConfigParser.ParsingError), e:
         raise ConfigError("Parsing error: %s" % str(e))
