@@ -20,44 +20,28 @@
 # the modules, but make no requirements on code importing these
 # modules.
 
+try:
+    set
+except NameError:
+    from sets import Set as set
+
 from linkexchange.utils import is_plugin_specifier, load_plugin
-from linkexchange.clients.base import ClientError, PageRequest
+from linkexchange.clients.base import ClientError, PageRequest, PageResponse
+
+class RawLink(object):
+    def __init__(self, link_code, client, client_no):
+        self.link_code = link_code
+        self.client = client
+        self.client_no = client_no
+
+    def __unicode__(self):
+        return self.link_code
 
 class Platform(object):
     """
     Platform combines various clients into one object. It have methods to get
     total list of raw links, to get links divided by blocks with custom
     formatting rules.
-
-    >>> from linkexchange.clients.sape import SapeTestServer
-    >>> srv = SapeTestServer()
-    >>> clients = [('sape', [], dict(user = 'user123456789',
-    ...    db_driver = ('mem',),
-    ...    use_xml = True, xml_server_list = [srv.url]))]
-    >>> pl = Platform(clients = clients)
-    >>> lx = pl.get_raw_links('http://example.com/')
-    >>> lx[0]
-    u'<a href="url1">link1</a>'
-    >>> lx[1]
-    u'<a href="url2">link2</a>'
-    >>> formatters = [('inline', [2], dict(
-    ...    class_ = 'links', class_for_empty = 'empty', suffix = '. ')),
-    ...    ('list', [2], dict(id = 'links'))]
-    >>> bx = pl.get_blocks('http://example.com/', formatters)
-    >>> bx[0]
-    u'<div class="links"><a href="url1">link1</a>. <a href="url2">link2</a>. </div>'
-    >>> bx[1]
-    u'<span id="links"></span>'
-    >>> bx = pl.get_blocks('http://example.com/notexists', formatters)
-    >>> bx[0]
-    u'<div class="empty"></div><!--12345-->'
-    >>> bx[1]
-    u'<span id="links"></span>'
-    >>> bx = pl.get_blocks('http://example.com/path/1', formatters)
-    >>> bx[0]
-    u'<div class="links"><a href="url1">link1</a>. <a href="url2">link2</a>. </div>'
-    >>> bx[1]
-    u'<ul id="links"><li><a href="url3">link3</a></li><li><a href="url4">link4</a></li></ul>'
     """
 
     def __init__(self, clients = None):
@@ -88,21 +72,25 @@ class Platform(object):
 
     def get_raw_links(self, request):
         """
-        Returns list of links, just like clients returns, but catches clients
-        errors and use HTML comments for errors reporting.
+        Returns list of links to place on page identified by request. Catches
+        clients errors and use HTML comments for errors reporting.
 
         @param request: PageRequest object or URL string
-        @return: list of links as unicode strings
+        @return: list of links as RawLink objects
         """
         if isinstance(request, basestring):
             request = PageRequest(url = request)
         result = []
+        cl_no = 0
         for cl in self.clients:
+            cl_no += 1
             try:
                 links = cl.get_raw_links(request)
             except ClientError, e:
                 links = [u'<!-- %s -->' % str(e)]
-            result.extend(links)
+            raw_links = [RawLink(link_code=l,
+                client=cl, client_no=cl_no) for l in links]
+            result.extend(raw_links)
         return result
 
     def get_blocks(self, request, formatters):
@@ -118,24 +106,53 @@ class Platform(object):
         @param formatters: sequence of blocks formatters
         @return: list of links blocks as unicode strings
         """
-        links = self.get_raw_links(request)
-        blocks = []
-
-        for fmt in formatters:
+        def load_formatter(fmt):
             if is_plugin_specifier(fmt):
-                fmt2 = load_plugin('linkexchange.formatters', fmt)
+                return load_plugin('linkexchange.formatters', fmt)
+            return fmt
+
+        def allocate_link(link, tag_list, link_list):
+            link = unicode(link)
+            if '<a ' in link:
+                link_list.append(link)
             else:
-                fmt2 = fmt
+                tag_list.append(link)
+
+        formatters = map(load_formatter, formatters)
+        links_pool = self.get_raw_links(request)
+        blocks = [None for fmt in formatters]
+
+        for i in range(len(formatters)):
+            fmt = formatters[i]
+            if fmt.client is None:
+                continue
+            if isinstance(fmt.client, basestring):
+                client_no_set = set([int(x.strip())
+                    for x in fmt.client.split(',')])
+            else:
+                client_no_set = set([fmt.client])
             tag_list = []
             link_list = []
-            while links and ((fmt2.count and
-                len(link_list) < fmt2.count) or (not fmt2.count)):
-                x = links.pop(0)
-                if '<a ' in x:
-                    link_list.append(x)
-                else:
-                    tag_list.append(x)
-            blocks.append(fmt2.format(tag_list, link_list))
+            j = 0
+            while j < len(links_pool) and ((fmt.count and
+                len(link_list) < fmt.count) or (not fmt.count)):
+                if links_pool[j].client_no in client_no_set:
+                    allocate_link(links_pool.pop(j), tag_list, link_list)
+                    continue
+                j += 1
+            blocks[i] = fmt.format(tag_list, link_list)
+
+        for i in range(len(formatters)):
+            if blocks[i] is not None:
+                continue
+            fmt = formatters[i]
+            tag_list = []
+            link_list = []
+            while links_pool and ((fmt.count and
+                len(link_list) < fmt.count) or (not fmt.count)):
+                allocate_link(links_pool.pop(0), tag_list, link_list)
+            blocks[i] = fmt.format(tag_list, link_list)
+
         return blocks
 
     def content_filter(self, request, content):
@@ -153,6 +170,22 @@ class Platform(object):
                 pass
         return content
 
+    def handle_request(self, request):
+        """
+        @param request: PageRequest object or URL string
+        @return: PageResponse object
+        """
+        if isinstance(request, basestring):
+            request = PageRequest(url=request)
+        for cl in self.clients:
+            try:
+                response = cl.handle_request(request)
+            except ClientError, e:
+                continue
+            if response.status != 404:
+                return response
+        return PageResponse(status=404)
+
     def refresh_db(self, request):
         """
         Force to refresh clients databases.
